/*====================================+=====================================+
! File CApp.h                         ! Copyright (C) 2018-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of kimfo.                                               !
! It is free software: you can redistribute it and/or modify it under the   !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/cmdline.h>
#include <wx/app.h>
#include <wx/filename.h>
#include <wx/snglinst.h>
#include "common/sc_base.h"
#include "common/sc_config.h"
/*-------------------------------------------------------------------------*/
class CFrame ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
enum e_server_type
{
   STY_KIMSUFI,
   STY_SYS_ESSENTIAL,
   STY_SYS_GAME,
   STY_NB
} ;

/*-------------------------------------------------------------------------*/
enum e_download_id
{
   DID_VERSION,
   DID_SERVER_TYPE_START,
   DID_SERVER_TYPE_END = DID_SERVER_TYPE_START + STY_NB - 1,
   DID_AVAIL_START,
   DID_AVAIL_END = DID_AVAIL_START + STY_NB - 1,
   DID_NB
} ;

/*-------------------------------------------------------------------------*/
struct str_sty
{  /*----------------------------------------------------------------------*/
   const char *m_p_c_name ;
   const char **m_p_p_c_xpm ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
struct str_region
{  /*----------------------------------------------------------------------*/
   const char *m_p_c_code ; // Code : fr, de etc.
   const char *m_p_c_name ; // Name : use wxString::FromUTF8 to display it
   const char *m_a_p_c_sty_page[ STY_NB ] ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CApp : public wxApp
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      ~CApp() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      bool       m_boo_first_start_ever = false ; // Program never started ?
      wxFileName m_fn_exe ; // Current executable
      wxLocale   m_locale ; // Default main locale object
      wxString   m_s_program_to_start_on_exit ;
      /*-------------------------------------------------------------------*/
      wxSingleInstanceChecker *m_sins_checker = nullptr ;
      sc::CSIServer *m_sins_server = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CFrame *m_frame = nullptr ;
      const str_region *m_p_region = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*--------------------------------------------------------------------+
      ! Serialized member variables : the "m" is upper-case                 !
      +--------------------------------------------------------------------*/
      CConfigRect   M_rec_main ;
      CConfigRect   M_rec_prefs ;
      CConfigRect   M_rec_progress ;
      CConfigRect   M_rec_lastversion ;
      CConfigRect   M_rec_newversion ;
      /*-------------------------------------------------------------------*/
      CConfigBool   M_boo_main_maxi ; // Main window maximized
      CConfigInt    M_i_spl_hor_pos ; // Horizontal splitter position
      /*-------------------------------------------------------------------*/
      CConfigString M_s_lang ; // Canonical name: en_US ...
      CConfigString M_s_region ; // Region code : see
      CConfigInt    M_i_chkupd_freq ;
      CConfigString M_s_chkupd_last ; // YYYY-MM-DD (ISO)
      CConfigBool   M_boo_startup_auto_check_avail ;
      CConfigBool   M_boo_startup_systray_only ;
      CConfigInt    M_i_check_avail_period ;
      CConfigString M_s_proxy_server ;
      CConfigInt    M_i_serv_list_sel ; // Selected notebook page
      CConfigString M_s_serv_ref_sel ; // Selected server references
      CConfigString M_s_serv_list_col_width ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool OnInit() override ;
      virtual void OnInitCmdLine( wxCmdLineParser &parser ) override ;
      virtual bool OnCmdLineParsed( wxCmdLineParser &parser ) override ;
      /*-------------------------------------------------------------------*/
      bool get_first_start_ever() const
      {  return m_boo_first_start_ever ; }
      int actions_for_install_process() ;
      wxString get_exe_path() const { return m_fn_exe.GetPath() ; }
      wxString get_exe_fullpath() const { return m_fn_exe.GetFullPath() ; }
      wxString get_exe_fullname() const { return m_fn_exe.GetFullName() ; }
      /*-------------------------------------------------------------------*/
      const wxString &get_program_to_start_on_exit() const
      {  return m_s_program_to_start_on_exit ; }
      void set_program_to_start_on_exit( const wxString &s_program )
      {  m_s_program_to_start_on_exit = s_program ; }
      void restart_on_exit()
      {  m_s_program_to_start_on_exit = get_exe_fullpath() ; }
      void no_start_on_exit()
      {  m_s_program_to_start_on_exit.clear() ; }
      /*-------------------------------------------------------------------*/
      wxString get_canonical_name() const
      {  return m_locale.GetCanonicalName() ; }
      void init_translations( const wxString &s_canonical_name
                                           = wxEmptyString
                            ) ;
      void init_config_info_to_keep() ;
      void open_config_file( const wxString &s_file, const int i_version ) ;
      void init_config_info() ;
      void save_config_info() ;
      /*-------------------------------------------------------------------*/
      const str_region *get_region( int i_num ) const ;
      const str_region *find_region( const wxString &s_region ) const ;
      void init_region() ;
      wxString get_region_code() const
      {  return m_p_region->m_p_c_code ; }
      wxString get_region_name() const
      {  return wxString::FromUTF8( m_p_region->m_p_c_name ) ; }
      wxString get_region_page( e_server_type sty ) const ;
      const str_sty *get_sty( e_server_type sty ) const ;
      const char *get_sty_name( e_server_type sty ) const
      {  return get_sty( sty )->m_p_c_name ; }
      const char **get_sty_xpm( e_server_type sty ) const
      {  return get_sty( sty )->m_p_p_c_xpm ; }
      /*-------------------------------------------------------------------*/
      int single_instance() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
DECLARE_APP( CApp )
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                            End of file CApp.h                             !
+==========================================================================*/
