/*====================================+=====================================+
! File sc_version.cpp                 ! Copyright (C) 2018-2020 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This is free software: you can redistribute it and/or modify it under the !
! terms of the GNU General Public License as published by the Free Software !
! Foundation, either version 3 of the License, or any later version.        !
! This application is distributed in the hope that it will be useful, but   !
! WITHOUT ANY WARRANTY; without even the implied warranty of                !
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General !
! Public License for more details.                                          !
! You should have received a copy of the GNU General Public License along   !
! with this application. If not, see <http://www.gnu.org/licenses/>.        !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/sstream.h>
#include <wx/txtstrm.h>
#include <wx/tokenzr.h>
#include "app_info.h"
#include "CApp.h"
#include "CFrame.h"
#include "common/sc_download.h"
#include "common/sc_version.h"
#include "common/sc_install.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
sc::CAvailableVersionData::CAvailableVersionData( const wxString &s_data )
{
   /*--( Parse version data )----------------------------------------------*/
   wxStringInputStream is( s_data ) ;
   wxTextInputStream   tis( is )    ;
   wxString            s_line       ;
   wxString            s_platform   ;
   wxString            s_url        ;

   /*--( The first line is the version number )----------------------------*/
   if( is.Eof() || ( m_s_version = tis.ReadLine() ).empty() )
   {  return ; }
   /*--( Then come the platforms/urls )------------------------------------*/
   while( !is.Eof() && !( s_line = tis.ReadLine() ).empty() )
   {  /*--( The two elements are separated by one or more spaces )---------*/
      s_platform = s_line.BeforeFirst( ' ', &s_url ) ;
      /*--( Remove leading spaces (the txt file is clearer ... ) )---------*/
      if( s_platform == APP_BUILT_TYPE )
      {  m_s_url_download = s_url.Trim().Trim( false ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*--( An empty line separates urls from version history )---------------*/
   if( is.Eof() ) { return ; }
   /*--( The rest is history ... )-----------------------------------------*/
   m_s_version_history = s_data.Mid( is.TellI() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::CDlgVersion::CDlgVersion( wxWindow *parent,
                              const wxString &s_main_info,
                              const sc::CAvailableVersionData &avd
                            )
               : wxDialog( parent, wxID_ANY, _( "Version" ),
                           wxDefaultPosition, wxDefaultSize,
                           wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                         ),
                 m_avd( avd )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   m_bt_web_site->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                        &CDlgNewVersion::OnBtGotoWebSite, this
                      ) ;
   m_bt_close->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                     &CDlgNewVersion::OnBtClose, this
                   ) ;
   /*----------------------------------------------------------------------*/
   m_stx_main_info->SetLabelText( s_main_info ) ;
   m_txt_version_history->SetValue( m_avd.m_s_version_history ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void sc::CDlgVersion::OnBtGotoWebSite( wxCommandEvent & WXUNUSED( event ) )
{
   /*----------------------------------------------------------------------*/
   wxLaunchDefaultBrowser( APP_URL_SITE ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int sc::CDlgVersion::get_modal()
{
   /*----------------------------------------------------------------------*/
   return ShowModal() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void sc::CDlgVersion::build_ui()
{
   /*----------------------------------------------------------------------*/
   SetSizeHints( wxSize( 600, 400 ), wxDefaultSize ) ;
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   /*----------------------------------------------------------------------*/
     m_stx_main_info
   = new wxStaticText( this, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE
                     ) ;
   m_stx_main_info->SetFont( m_stx_main_info->GetFont().Scale( 2 ).Bold() ) ;
   bsz_gen->Add( m_stx_main_info, 0, wxEXPAND | wxTOP | wxBOTTOM, 20 ) ;
   /*----------------------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_hist
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Version history" ) ) ;
   wxStaticBox *stb_hist = stbsz_hist->GetStaticBox() ;
   m_txt_version_history
   = new wxTextCtrl( stb_hist, wxID_ANY, wxEmptyString,
                     wxDefaultPosition, wxDefaultSize,
                     wxHSCROLL | wxTE_MULTILINE | wxTE_READONLY
                   ) ;
   stbsz_hist->Add( m_txt_version_history, 1, wxEXPAND, 5 ) ;
   bsz_gen->Add( stbsz_hist, 1, wxLEFT | wxRIGHT | wxEXPAND, 5 ) ;
   /*----------------------------------------------------------------------*/
   m_bsz_btn = new wxBoxSizer( wxHORIZONTAL ) ;

   m_bt_web_site = new wxButton( this, wxID_ANY, _( "&Web site" ) ) ;
   m_bsz_btn->Add( m_bt_web_site, 0, wxLEFT | wxRIGHT, 5 ) ;

   m_bt_close = new wxButton( this, wxID_CLOSE, wxEmptyString ) ;
   m_bt_close->SetDefault() ;
   m_bsz_btn->Add( m_bt_close, 0, wxLEFT | wxRIGHT, 5 ) ;

   bsz_gen->Add( m_bsz_btn, 0, wxALIGN_RIGHT | wxALL, 10 ) ;
   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::CDlgLastVersion::CDlgLastVersion( wxWindow *parent,
                                      const wxString &s_main_info,
                                      const sc::CAvailableVersionData &avd
                                    )
                   : sc::CDlgVersion( parent, s_main_info, avd )
{
   /*----------------------------------------------------------------------*/
   sc::init_tlw_rect( this, wxGetApp().M_rec_lastversion ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
   m_bt_close->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::CDlgLastVersion::~CDlgLastVersion()
{
   /*----------------------------------------------------------------------*/
   sc::save_tlw_rect( this, wxGetApp().M_rec_lastversion ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::CDlgNewVersion::CDlgNewVersion( wxWindow *parent,
                                    const wxString &s_main_info,
                                    const sc::CAvailableVersionData &avd
                                  )
                  : sc::CDlgVersion( parent, s_main_info, avd )
{
   /*----------------------------------------------------------------------*/
   m_bt_download = new wxButton( this, wxID_ANY, _( "&Download" ) ) ;
   m_bsz_btn->Prepend( m_bt_download, 0, wxLEFT | wxRIGHT, 5 ) ;

     m_bt_download_install
   = new wxButton( this, wxID_ANY, _( "Download && &Install" ) ) ;
   m_bt_download_install->SetDefault();
   m_bsz_btn->Prepend( m_bt_download_install, 0, wxLEFT | wxRIGHT, 5 ) ;

   /*----------------------------------------------------------------------*/
   m_bt_download_install->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                                &CDlgNewVersion::OnBtDownloadInstall, this
                              ) ;
   m_bt_download->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                        &CDlgNewVersion::OnBtDownload, this
                      ) ;
   /*----------------------------------------------------------------------*/
   sc::init_tlw_rect( this, wxGetApp().M_rec_newversion ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
   m_bt_download_install->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
sc::CDlgNewVersion::~CDlgNewVersion()
{
   /*----------------------------------------------------------------------*/
   sc::save_tlw_rect( this, wxGetApp().M_rec_newversion ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void sc::CDlgNewVersion::OnBtDownload( wxCommandEvent &event )
{
   /*--( After download only a "wait" is done )----------------------------*/
   download( wxFileName(), CDlgDownload::DDO_END_WAIT ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void sc::CDlgNewVersion::OnBtDownloadInstall( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   if( wxMessageBox( _( "Start download and installation ?" ),
                     _( "Confirmation" ), wxYES_NO, GetParent()
                   ) != wxYES
     )
   {  return ; }
   /*----------------------------------------------------------------------*/
   wxFileName fn_exe_path = wxFileName::DirName( wxGetApp().get_exe_path() );
   /*--( By default the new version is downloaded in the executable path )-*/
   if( download( fn_exe_path ) == CDownloadBase::DE_NO_ERROR_DONE )
   {
       /*--( If the installation failed no restart is needed )-------------*/
       if( install() != sc::CInstallThread::IE_NO_ERROR_DONE )
       {  wxGetApp().no_start_on_exit() ; }
       else
       {  EndModal( 0 ) ; }
       /*------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int sc::CDlgNewVersion::download( const wxFileName &fn_local,
                                  CDlgDownload::e_option ddo_option
                                )
{
   /*----------------------------------------------------------------------*/
   if( m_avd.m_s_url_download.empty() )
   {  wxLogError( _( "The URL for your system has not been found (%s)" ),
                  APP_BUILT_TYPE
                ) ;
      return -1 ;
   }
   /*--( Use specified directory of ask the user to choose it )------------*/
   if( fn_local.IsOk() )
   {  m_fn_local = fn_local ; }
   else
   {  /*--( Choose directory/name )----------------------------------------*/
      wxFileDialog dlg_fd( this, "", "",
                           m_avd.m_s_url_download.AfterLast( '/' ),
                           wxFileSelectorDefaultWildcardStr,
                           wxFD_SAVE | wxFD_OVERWRITE_PROMPT
                         ) ;
      if( dlg_fd.ShowModal() == wxID_CANCEL )
      {  return -2 ; }
      /*-------------------------------------------------------------------*/
      m_fn_local = dlg_fd.GetPath() ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Lets go )---------------------------------------------------------*/
   CDlgDownload dlg( this, m_avd.m_s_url_download, m_fn_local, ddo_option ) ;
   m_fn_local = dlg.get_dlbase().build_local_fullpath() ;
   return dlg.go() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int sc::CDlgNewVersion::install()
{
   /*----------------------------------------------------------------------*/
   return sc::CDlgInstall( this, m_fn_local ).go() ;
   /*----------------------------------------------------------------------*/
}

/*--( Fill choice normally used in the preferences dialog )----------------*/
void sc::fill_cho_chkupd_freq( wxChoice *cho_chkupd_freq )
{
   /*----------------------------------------------------------------------*/
   cho_chkupd_freq->Append( _( "never" ) ) ;   // CUF_NEVER
   cho_chkupd_freq->Append( _( "daily" ) ) ;   // CUF_DAY
   cho_chkupd_freq->Append( _( "weekly" ) ) ;  // CUF_WEEK
   cho_chkupd_freq->Append( _( "monthly" ) ) ; // CUF_MONTH
   /*----------------------------------------------------------------------*/
}

/*--( Find date for the next update check : empty means not to be checked )*/
wxString sc::next_chkupd_iso( const wxString &s_last, int i_freq )
{
   /*----------------------------------------------------------------------*/
   if( i_freq <= CUF_NEVER || i_freq >= CUF_NB )
   {  return wxEmptyString ; }
   /*----------------------------------------------------------------------*/
   const wxDateTime co_dt_today = wxDateTime::Today() ;
   wxDateTime       dt_next     = co_dt_today ;
   /*--( Never checked or invalid stored date means today )----------------*/
   if( !dt_next.ParseISODate( s_last ) )
   {  return dt_next.FormatISODate() ; }
   /*----------------------------------------------------------------------*/
   switch( i_freq )
   {
      /*-------------------------------------------------------------------*/
      case CUF_DAY   : dt_next.Add( wxDateSpan( 0, 0, 0, 1 ) ) ; break ;
      case CUF_WEEK  : dt_next.Add( wxDateSpan( 0, 0, 1, 0 ) ) ; break ;
      case CUF_MONTH : dt_next.Add( wxDateSpan( 0, 1, 0, 0 ) ) ; break ;
      /*-------------------------------------------------------------------*/
   }
   /*-----------------------------------------------------------------------+
   ! The next check can't be in the past ...                                !
   ! It can be so, for ex., if "last" <= today - 2 and "freq" = CUF_DAY     !
   +-----------------------------------------------------------------------*/
   if( dt_next < co_dt_today ) { dt_next = co_dt_today ; }
   /*----------------------------------------------------------------------*/
   return dt_next.FormatISODate() ;
   /*----------------------------------------------------------------------*/
}

/*--( The string date is in ISO format )-----------------------------------*/
bool sc::check_version_needed( const wxString &s_last, int i_freq )
{
   /*----------------------------------------------------------------------*/
   const wxString co_s_today = wxDateTime::Today().FormatISODate() ;
   const wxString co_s_next_chkupd = sc::next_chkupd_iso( s_last, i_freq ) ;
   /*----------------------------------------------------------------------*/
   return !co_s_next_chkupd.empty() && co_s_next_chkupd <= co_s_today ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void sc::check_version( int i_err_code,
                        const wxString &s_data,
                        bool boo_manual_check
                      )
{
   /*----------------------------------------------------------------------*/
   wxString s_message ;
   /*-----------------------------------------------------------------------+
   ! Even if the last version data could not be retrieved the check is      !
   ! considered as done                                                     !
   +-----------------------------------------------------------------------*/
   wxGetApp().M_s_chkupd_last.set( wxDateTime::Today().FormatISODate() ) ;

   /*----------------------------------------------------------------------*/
   sc::CAvailableVersionData avd( s_data ) ;

   /*--( Error ? )---------------------------------------------------------*/
   if(    i_err_code != sc::CDownloadBase::DE_NO_ERROR_DONE
       || avd.m_s_version.empty()
     )
   {  /*-------------------------------------------------------------------*/
      if( boo_manual_check )
      {  /*----------------------------------------------------------------*/
         s_message.Printf( _( "%s's last version data could not be "
                              "downloaded:\n%s"
                            ),
                           APP_DISPLAY_NAME,
                           sc::CDownloadBase::st_get_err_msg( i_err_code )
                         ) ;
         wxMessageBox( s_message ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   CFrame *frame = wxGetApp().m_frame ;
   /*--( Same old ? )------------------------------------------------------*/
   if( APP_VERSION == avd.m_s_version )
   {
      /*-------------------------------------------------------------------*/
      if( boo_manual_check )
      {  /*----------------------------------------------------------------*/
         s_message = _( "This version is the latest available" ) ;
         sc::CDlgLastVersion( frame, s_message, avd ).get_modal() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Binary available for this system ? )------------------------------*/
   if( avd.m_s_url_download.empty() )
   {
      /*-------------------------------------------------------------------*/
      if( boo_manual_check )
      {  /*----------------------------------------------------------------*/
         s_message.Printf( _( "Version %s is available but not\n"
                              "for your system (%s)"
                            ),
                           avd.m_s_version, APP_BUILT_TYPE
                         ) ;
         sc::CDlgLastVersion( frame, s_message, avd ).get_modal() ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
      return ;
      /*-------------------------------------------------------------------*/
   }

   /*--( New version ! )---------------------------------------------------*/
   s_message.Printf( _( "The version %s is available" ), avd.m_s_version ) ;
   sc::CDlgNewVersion( frame, s_message, avd ).get_modal() ;
   /*--( After new version installed the program is restarted )------------*/
   if( !wxGetApp().get_program_to_start_on_exit().empty() )
   {  frame->Close( true ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                        End of file sc_version.cpp                         !
+==========================================================================*/
